# -*- coding: UTF-8 -*-

import os
from os import path
from lang import ids

__CACHE_NAME_SIZE = 6


def get_temp_path():
    temp_path = '/tmp'
    if not path.exists(temp_path):
        temp_path = path.join(os.getenv('HOME'), 'temp')
        if not path.exists(temp_path):
            os.mkdir(temp_path)
    return temp_path


def get_cache_path():
    temp_path = path.join(get_temp_path(), ids.generate(__CACHE_NAME_SIZE))
    if not path.exists(temp_path):
        os.mkdir(temp_path)

    return temp_path



# -*- coding: UTF-8 -*-

import os


class Properties:
    """
    读取Properties 文件类
    """

    def __init__(self, file_name):
        with open(file_name, 'r', encoding='utf-8') as pro_file:
            for line in pro_file:
                if line.find('=') > 0:
                    array = line.replace('\n', '').split('=')
                    self.__dict__[array[0].strip()] = array[1].strip()

    def dict(self):
        return self.__dict__


def load_properties(abspath):
    return Properties(abspath).dict()


def read(abspath):
    text = ''
    with open(abspath, 'r') as file:
        text = file.read()

    return text


def test_properties():
    p = Properties(os.path.join(os.environ['HOME'], '.pycli'))
    print(p.dict())



# -*- coding: UTF-8 -*-

import os
from os import path

from androguard.core.bytecodes.apk import APK

import xcli
from xcli import ArgOpts
from xcli import OPT_DEPLOY
from xcli import oxs, apks, fir
from xcli.fir import FirToken


def get_apk_channel_abspath(module_abspath):
    return path.join(module_abspath, 'channel.ini')


def check_channel_config(channel_abspath):
    return path.exists(channel_abspath) and path.getsize(channel_abspath)


def check_project(args: ArgOpts):
    return path.exists(path.join(args.project_path, args.app_module, 'build.gradle'))


def deploy(args: ArgOpts):
    module_abspath = path.join(args.project_path, args.app_module)
    if path.exists(module_abspath):
        config = xcli.load_config()

        apks.gradle_build(args.project_path, args.app_module)
        print('\n[INFO]: 工程编译完成：%s' % args.project_path)

        flavors_name = apks.get_flavors_name(module_abspath)
        build_apk_name = apks.get_build_apk_name(args.app_module)

        build_outputs_abspath = apks.get_build_output_path(module_abspath, flavors_name)
        build_apk_abspath = path.join(build_outputs_abspath, build_apk_name)
        if path.exists(build_apk_abspath):
            apk_object = APK(build_apk_abspath)
            apk_deploy_name = apks.get_deploy_name(apk_object)
            app_code = apks.get_app_code(apk_object)
            app_version_name = apk_object.get_androidversion_name()

            oss_key = path.join(app_code, flavors_name, app_version_name, apk_deploy_name)
            verify_result = oxs.publish(oss_key, build_apk_abspath)
            if verify_result:
                print('[INFO]: apk文件上传OSS成功！')

                # outputs apk文件修改成正式发布的名称
                apk_deploy_abspath = path.join(build_outputs_abspath, apk_deploy_name)
                os.rename(build_apk_abspath, apk_deploy_abspath)

                manifest = apks.Manifest(apk_object)
                manifest.deploy_abspath = apk_deploy_abspath

                # 上传到Fir.im上
                fir_cert = fir.validate(FirToken(config.fir_token, apk_object.get_package()))
                is_completed = fir.publish(fir_cert, manifest)
                if is_completed:
                    print('[INFO]: apk文件上传FIR.IM成功！')

            else:
                raise RuntimeError("\n[ERROR]: oss上文件和本地apk文件校验失败！")
        elif apks.check_unsigned_apk_built(build_outputs_abspath, args.app_module):
            raise RuntimeError("\n[ERROR]: apk文件未签名：%s！" % build_apk_abspath)

        else:
            raise RuntimeError("\n[ERROR]: 工程编译失败，未生成apk文件：%s！" % build_apk_abspath)
    else:
        raise RuntimeError('\n[ERROR]: 工程文件不存在：%s！' % module_abspath)


def main():
    args = ArgOpts()
    print(args)

    if args.operate == OPT_DEPLOY:
        if check_project(args):
            deploy(args)

        else:
            raise RuntimeError('\n[ERROR]: 当前目录非Android工程目录，请指定编译目录！！！')


if __name__ == "__main__":
    main()



# -*- coding: UTF-8 -*-

import string
import random

__SHORT_NAME_SIZE = 6
__ASCII_CHARACTER = string.ascii_letters + string.digits


def generate(length=__SHORT_NAME_SIZE):
    return ''.join(random.sample(__ASCII_CHARACTER, length))



# -*- coding: UTF-8 -*-

import json

import requests
from androguard.core.bytecodes.apk import APK

from xcli import PyCliConfig, apks


class FirToken:
    token = ''
    application_id = ''

    def __init__(self, t: str, app_id: str):
        self.token = t
        self.application_id = app_id


class FirCertEntry:
    class CertMeta:
        key = ''
        token = ''
        upload_url = ''

        def __init__(self, cert_json: json):
            self.key = cert_json['key']
            self.token = cert_json['token']
            self.upload_url = cert_json['upload_url']

    type = ''
    short = ''
    icon: CertMeta
    binary: CertMeta

    prefix = ''

    def __init__(self, result_json):
        self.type = result_json['type']
        self.short = result_json['short']

        cert_json = result_json['cert']

        self.icon = self.CertMeta(cert_json['icon'])
        self.binary = self.CertMeta(cert_json['binary'])
        self.prefix = cert_json['prefix']


# 获取 fir 的上传凭证
def validate(token: FirToken):
    params = {'type': 'android',
              'bundle_id': token.application_id,
              'api_token': token.token}

    response_result = requests.post('http://api.jappstore.com/apps', data=params)

    # 拿到cert实体
    cert_json = json.loads(response_result.content)
    if 'code' in cert_json:
        raise RuntimeError('[ERROR]: fir认证失败：%d：%s' % (cert_json['code'], cert_json['errors']['exception']))

    return FirCertEntry(cert_json)


# 上传到fir
def publish(fir_cert: FirCertEntry, manifest: apks.Manifest):
    file = {'file': open(manifest.deploy_abspath, 'rb')}
    params = {
        "key": fir_cert.binary.key,
        "token": fir_cert.binary.token,
        "x:name": manifest.app_name,
        "x:version": manifest.version_name,
        'x:build': manifest.version_code,
        "x:changelog": manifest.change_log
    }

    requests.packages.urllib3.disable_warnings()
    req = requests.post(fir_cert.binary.upload_url, files=file, data=params, verify=False)

    update_result = json.loads(req.content)

    return update_result['is_completed']


def test_fir_publish():
    config = PyCliConfig()

    _UPDATE_FILE_PATH = '/Users/handy/datum/workspace/t/EmptyApp/app/build/outputs/apk/release/app-release.apk'
    apk = APK(_UPDATE_FILE_PATH)

    fir_cert = validate(FirToken(config.fir_token, apk.get_package()))

    manifest = apks.Manifest(apk)
    manifest.deploy_abspath = _UPDATE_FILE_PATH
    _is_completed = publish(fir_cert, manifest)

    print('[INFO]: test_fir_publish: %s' % _is_completed)



# -*- coding: UTF-8 -*-

import oss2

import xcli
from xcli import PyCliConfig


def get_bucket_name(config: PyCliConfig):
    service = oss2.Service(config.auth, config.endpoint)
    bucket_list = [b.name for b in oss2.BucketIterator(service)]

    return bucket_list


def get_bucket(config: PyCliConfig):
    return oss2.Bucket(config.auth, config.endpoint, config.bucket)


def publish(oss_key, apk_abspath):
    """
    上传apk文件到OSS上

    :param oss_key: oss key
    :param apk_abspath: apk绝对路径
    :return:
    """
    # 上传OSS
    config = xcli.load_config()
    config.auth = oss2.Auth(config.access_key_id, config.access_key_secret)
    oss_bucket = get_bucket(config)

    oss_bucket.put_object_from_file(oss_key, apk_abspath)
    with open(apk_abspath, 'rb') as apk_file_object:
        # 验证上传的文件与本地文件是否一致
        verify_result = oss_bucket.get_object(oss_key).read() == apk_file_object.read()

    return verify_result



# -*- coding: UTF-8 -*-
import os
import sys
import threading
from getopt import getopt
from os import path, environ

from oss2 import Auth

from lang.io import Properties

__title__ = 'py-cli'
__version__ = '1.2.4'

__PYCLI_PATH = path.abspath(path.join(path.dirname(os.getcwd()), '.'))
__LIBS_PATH = path.join(__PYCLI_PATH, 'libs')

OPT_DEPLOY = 'deploy'
OPT_ENCRYPT = 'encrypt'
OPT_CHANNEL = 'channel'


class ArgOpts(object):
    operate = OPT_DEPLOY

    project_path = ''
    app_module = ''

    output_path = ''
    channel_config_path = ''

    def __init__(self):
        self.operate = sys.argv[1]
        if self.operate in ("-v", "--version"):
            print(__version__)
            sys.exit(0)

        elif self.operate in ("-h", "--help"):
            print_usage()
            sys.exit(0)

        else:
            self.parse_args()

    def parse_args(self):
        opts, args = getopt(sys.argv[2:],
                            'p:m:c:o:',
                            ['project=' 'module=', 'channel=', 'output='])
        if opts:
            for opt, arg in opts:
                if opt in ("-p", "--project"):
                    if arg:
                        self.project_path = arg

                elif opt in ("-m", "--module"):
                    if arg:
                        self.app_module = arg
                    else:
                        raise RuntimeError('[ERROR]: Usage: pycli -m <your project path> | --project="your project path"')

                elif opt in ("-c", "--channel"):
                    self.channel_config_path = arg

                elif opt in ("-o", "--output"):
                    self.output_path = arg

        if len(self.project_path) <= 0:
            self.project_path = os.getcwd()

        if not self.app_module:
            self.app_module = 'app'

        if not self.output_path:
            self.output_path = os.getcwd()

    def __str__(self):
        return """--------------------------------------
[INFO]:
  operate         = %s
  project_path    = %s
  app_module      = %s
  output_path     = %s
  channel_config  = %s
--------------------------------------
""" % (self.operate, self.project_path, self.app_module, self.output_path, self.channel_config_path)


class PyCliConfig(object):
    """
    PyCli配置，其配置文件保存在~/.pycli

    endpoint=http://oss-cn-hangzhou.aliyuncs.com
    access_key_id=<your oss access key id>
    access_key_secret=<your oss access key secret>
    bucket=apk-res
    fir_token=<fir.in token>
    """
    __instance_lock = threading.Lock()

    props = {}

    endpoint = 'http://oss-cn-hangzhou.aliyuncs.com'
    access_key_id = ''
    access_key_secret = ''
    bucket = 'apk-res'
    fir_token = ''

    auth: Auth = None

    def __init__(self):
        self.props = Properties(path.join(environ['HOME'], '.pycli')).dict()

        self.access_key_id = self.props.get('access_key_id')
        self.access_key_secret = self.props.get('access_key_secret')
        self.bucket = self.props.get('bucket')
        self.endpoint = self.props.get('endpoint')

        self.fir_token = self.props.get('token')

    def __new__(cls, *args, **kwargs):
        if not hasattr(PyCliConfig, "_instance"):
            with PyCliConfig.__instance_lock:
                if not hasattr(PyCliConfig, "_instance"):
                    PyCliConfig._instance = object.__new__(cls)
        return PyCliConfig._instance


def load_config():
    return PyCliConfig()


def get_pycli_path():
    return __PYCLI_PATH


def get_libs_path():
    return __LIBS_PATH


def print_usage():
    print('Usage: python -m <app_module>')


__all__ = ['__title__', '__version__', 'OPT_DEPLOY', 'PyCliConfig', 'ArgOpts']



# -*- coding: UTF-8 -*-

import os
import shutil
import subprocess
from os import path

from androguard.core.bytecodes.apk import APK

import xcli
from lang import files
from lang import ids
from lang.io import Properties

__PYCLI_PATH = path.abspath(path.join(path.dirname(os.getcwd()), '.'))
__LIBS_PATH = path.join(__PYCLI_PATH, 'libs')
__CHECK_SIGNATURE_V2_PATH = path.join(__LIBS_PATH, "CheckAndroidV2Signature.jar")

__WALLE_CLI_PATH = path.join(xcli.get_libs_path(), "walle-cli-all.jar")

__GENERAL_FLAVORS_NAME = 'apk'

__ASSEMBLE_RELEASE = 'assembleRelease'
__DEPLOY_RELEASE = 'release'
__NAMESPACE_XRJS = 'xrjs'


class Manifest(object):
    app_name = ''
    version_name = '1.0'
    version_code: int = 1
    change_log = '版本更新'

    deploy_abspath = ''

    def __init__(self, apk_object: APK):
        self.app_name = apk_object.get_app_name()
        self.version_name = apk_object.get_androidversion_name()
        self.version_code = apk_object.get_androidversion_code()
        self.change_log = '版本更新 %s' % self.version_name


class JksConfig(object):
    """
    读取key_store配置文件

    key.store.path=/datum/workspace/x/profile/xrj_app.jks
    key.store.password=xxxxx
    key.alias=xiot
    key.password=xxxxx
    """
    props = {}

    store_path = ''
    store_password = ''
    key_alias = ''
    key_password = ''

    def __init__(self, abspath):
        self.props = Properties(abspath).dict()

        self.store_path = self.props.get('key.store.path')
        self.store_password = self.props.get('key.store.password')
        self.key_alias = self.props.get('key.alias')
        self.key_password = self.props.get('key.password')


def get_app_code(apk: APK):
    """
    获取apk中登记的app code

    :param apk:
    :return:
    """
    _manifest_xml = apk.get_android_manifest_xml()
    manifest_xml = apk.get_android_manifest_xml()

    ns_xrjs = manifest_xml.nsmap[__NAMESPACE_XRJS]
    application_element = manifest_xml.xpath('/manifest/application')[0]
    app_code = application_element.attrib["{%s}appCode" % ns_xrjs]

    return app_code


def get_deploy_name(apk: APK):
    """
    获取默认发布的apk名称

    :param apk:
    :return:
    """
    return '%s_%s_build_%s.apk' % (get_app_code(apk), apk.get_androidversion_name(), apk.get_androidversion_code())


def get_build_apk_name(app_module, deploy_type=__DEPLOY_RELEASE):
    return "%s-%s.apk" % (app_module, deploy_type)


def get_build_output_path(module_abspath, flavors_name, deploy_type=__DEPLOY_RELEASE):
    return path.join(module_abspath, 'build/outputs', flavors_name, deploy_type)


def has_product_flavors(file_path):
    has_flavors = False

    with open(file_path) as file:
        line_text = file.readline()
        while line_text:
            if 'productFlavors' in line_text:
                has_flavors = True
                break

            line_text = file.readline()

    return has_flavors


def get_flavors_name(project_module_path):
    flavors_name = __GENERAL_FLAVORS_NAME

    build_gradle_path = os.path.join(project_module_path, 'build.gradle')

    has_flavors = has_product_flavors(build_gradle_path)
    if has_flavors:
        pass

    return flavors_name


def check_unsigned_apk_built(build_outputs_path, app_module, deploy_type=__DEPLOY_RELEASE):
    unsigned_apk_name = "%s-%s-unsigned.apk" % (app_module, deploy_type)

    return os.path.exists(os.path.join(build_outputs_path, deploy_type, unsigned_apk_name))


def gradle_build(project_path, app_module):
    """
    执行gradle脚本

    :param project_path:
    :param app_module:
    :return:
    """
    build_command = "./gradlew -q -p %s clean assembleRelease" % app_module
    print('[INFO]: %s: %s' % (build_command, project_path))
    subprocess.run(build_command, shell=True, cwd=project_path)


def get_jks_config(abspath):
    gradle_configs = Properties(abspath).dict()
    jks_abspath = gradle_configs.get('KEYSTORE_PROPERTIES_PATH')
    if jks_abspath:
        jks_config = JksConfig(jks_abspath)
    else:
        raise RuntimeError("\n[ERROR]: Apk签名配置文件加载失败：%s！" % jks_abspath)

    return jks_config


def zipalign(apk_abspath, out_path):
    """
    zipalign对齐

    :param apk_abspath:
    :param out_path:
    :return:
    """
    aligned_apk_path = path.join(out_path, "%s_aligned.apk" % ids.generate())
    zipalign_command = "zipalign -v 4 %s %s" % (apk_abspath, aligned_apk_path)
    print(zipalign_command)
    subprocess.run(zipalign_command, shell=True)

    return aligned_apk_path


def sign(apk_abspath, jks: JksConfig, out_path):
    """
    签名和校验V2
    :param apk_abspath:
    :param jks:
    :param out_path:
    :return:
    """
    signed_apk_path = path.join(out_path, "%s_signed.apk" % ids.generate())
    apk_signer_command = "apksigner sign --ks %s --ks-pass pass:%s --ks-key-alias %s --key-pass pass:%s --out %s %s" \
                         % (jks.store_path, jks.store_password, jks.key_alias, jks.key_password, signed_apk_path, apk_abspath)
    print('[INFO]: %s' % apk_signer_command)
    subprocess.run(apk_signer_command, shell=True)

    # 检查V2签名是否正确
    check_signature_v2_command = "java -jar %s %s" % (__CHECK_SIGNATURE_V2_PATH, signed_apk_path)
    print('[INFO]: %s' % check_signature_v2_command)
    subprocess.run(check_signature_v2_command, shell=True)

    return signed_apk_path


def channel(apk_abspath, channel_abspath, output_path):
    """
    写入渠道

    :param apk_abspath:
    :param channel_abspath:
    :param output_path:
    :return:
    """
    walle_cli_command = "java -jar %s batch -f %s %s %s" % (__WALLE_CLI_PATH, channel_abspath, apk_abspath, output_path)
    print(walle_cli_command)
    subprocess.run(walle_cli_command, shell=True)


def assemble(apk_abspath, jks: JksConfig, channel_abspath, output_path):
    walle_temp_path = files.get_cache_path()

    # 对齐
    aligned_apk_path = zipalign(apk_abspath, walle_temp_path)

    # 签名
    signed_apk_path = sign(aligned_apk_path, jks, walle_temp_path)

    # 写入渠道
    channel(signed_apk_path, channel_abspath, output_path)

    shutil.rmtree(walle_temp_path)



# -*- coding: UTF-8 -*-

from setuptools import setup, find_packages

import re
from os import path


def get_property(prop, project):
    result = re.search(r'{}\s*=\s*[\'"]([^\'"]*)[\'"]'.format(prop), open(path.join(project, '__init__.py')).read())

    return result.group(1)


with open('README.md', encoding='utf-8') as readme_file:
    _long_description = readme_file.read()

setup(
    author='handy',
    author_email='cvdnn@foxmail.com',
    url='https://pypi.org/project/py-cli/',

    license='Apache v2.0',
    description='git-runner cli script',
    long_description=_long_description,
    long_description_content_type='text/markdown',
    keywords=["gitlab_runner_cli", "android_cli"],

    name='py-cli',
    version=get_property('__version__', 'xcli'),

    packages=find_packages(),
    include_package_data=True,
    platforms="any",
    install_requires=['androguard', 'oss2', 'requests'],
    extras_require={  # Optional
        'dev': ['check-manifest'],
        'test': ['coverage'],
    },

    entry_points={
        'console_scripts': [
            'pycli = xcli.invoke:main'
        ],
    },

    # 此项需要，否则卸载时报windows error
    zip_safe=False
)
